Package: kohonen
Version: 3.0.13
Title: Supervised and Unsupervised Self-Organising Maps
Authors@R: c(person(given = "Ron",
                      family = "Wehrens",
                      role = c("aut", "cre"),
                      email = "ron.wehrens@gmail.com"),
               person(given = "Johannes",
                      family = "Kruisselbrink",
                      role = "aut"))
Maintainer: Ron Wehrens <ron.wehrens@gmail.com>
Description: Functions to train self-organising maps (SOMs). Also interrogation of the maps and prediction using trained maps are supported. The name of the package refers to Teuvo Kohonen, the inventor of the SOM.
License: GPL (>= 2)
Depends: R (>= 2.10)
Imports: Rcpp (>= 0.12.12)
Suggests: RColorBrewer, lattice, vegan
LinkingTo: Rcpp
NeedsCompilation: yes
Packaged: 2026-01-22 10:13:51 UTC; ron
Author: Ron Wehrens [aut, cre],
  Johannes Kruisselbrink [aut]
Repository: CRAN
Date/Publication: 2026-01-22 10:50:02 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-26 11:57:27 UTC; unix
Archs: kohonen.so.dSYM
